# boek/omroepers/doen.py
#
#

""" basic package for the program. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from boek.definitie import BLA, GREEN, ENDC
from boek.hemel import hemel, vloot, regels
from boek.regels import regels_doen
from boek.helpers import stripped
from boek.omroepers import Omroeper
from boek import Ding

import threading
import logging
import getpass
import queue
import time
import html

## XMPP

class Doen(Omroeper):

    """ De Doen Omroeper. """

    cc = ""

    def __init__(zelf, *args, **kwargs):
        import sleekxmpp
        from sleekxmpp import clientxmpp
        Omroeper.__init__(zelf, *args, **kwargs)
        if "port" not in zelf: zelf.port = 5222
        zelf._queue = queue.Queue()
        zelf.doen = clientxmpp.ClientXMPP(zelf.user, getpass.getpass())
        zelf.doen.add_event_handler("session_start", zelf.session_start)
        zelf.doen.add_event_handler("message", zelf.handle_message)
        zelf.doen.add_event_handler('disconnected', zelf.handle_disconnected)
        zelf.doen.add_event_handler('connected', zelf.handle_connected)
        zelf.doen.add_event_handler('presence_available', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_dnd', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_xa', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_chat', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_away', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_unavailable', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_subscribe', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_subscribed', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_unsubscribe', zelf.handle_presence)
        zelf.doen.add_event_handler('presence_unsubscribed', zelf.handle_presence)
        zelf.doen.add_event_handler('groupchat_message', zelf.handle_message)
        zelf.doen.add_event_handler('groupchat_presence', zelf.handle_presence)
        zelf.doen.add_event_handler('groupchat_subject', zelf.handle_presence)
        zelf.doen.add_event_handler('failed_auth', zelf.handle_failedauth)
        zelf.doen.exception = zelf.exception
        zelf.doen.use_signals()
        zelf.openfire = regels.openfire
        if zelf.openfire:
            logging.warn("# Doen/openfire")
            import ssl
            zelf.doen.ssl_version = ssl.PROTOCOL_SSLv3
        zelf._connected = threading.Event()
        zelf.channels = []

    def _raw(zelf, *args, **kwargs):
        logging.debug("> Doen/schrijf %s" % args[0])
        zelf.doen.send_raw(args[0])

    def start(zelf, *args, **kwargs):
        zelf.connect()
        Omroeper.start(zelf, *args, **kwargs)

    def connect(zelf):
        logging.warn("# Doen/verbind met %s" % zelf.server)
        try: zelf.doen.connect((zelf.server, zelf.port), use_ssl=zelf.openfire)
        except: zelf.doen.connect((zelf.server, zelf.port))

    def session_start(zelf, event):
        zelf.doen.send_presence()
        zelf.ready()
        logging.warn("# Doen/sessie %s" % zelf.user)
         
    def exception(zelf, ex):
        zelf._status = ex
        logging.error("^ Doen/fout %s"% str(ex))

    def melden(zelf, *args, **kwargs):
        for channel in zelf.channels: zelf.say(channel, args[0])

    def handle_failedauth(zelf, error, *args, **kwargs): zelf._error = error ; logging.error("^ Doen/authoriseer %s" % str(error))

    def handle_failure(zelf, error, *args, **kwargs): zelf._error = error ; logging.error("^ Doen/fout %s" % str(error))

    def handle_disconnected(zelf, *args, **kwargs):
        zelf._connected.clear()
        zelf._status = "opgehangen"
        logging.error("^ Doen/opgehangen %s" % zelf.user)

    def handle_connected(zelf, *args, **kwargs): 
        zelf._status = "verbonden"
        zelf._connected.set()
        logging.warn("# Doen/verbonden %s" % zelf.user)

    def loop(zelf, *args, **kwargs): zelf.doen.process(block=True)

    def say(zelf, *args, **kwargs):
        to = args[0]
        txt = args[1]
        zelf.doen.send_message(to, txt)
        logging.info("> Doen/bericht %s" % to)

    def event(zelf, *args, **kwargs):
        event = zelf._queue.get()
        event.cc = zelf.cc
        return event

    def handle_message(zelf, data, *args, **kwargs):
        logging.info("< Doen/lees %s" % data)
        m = Ding(**data)
        if m.type == "error": logging.error("^ %s" % m.error) ; return
        m["from"] = str(m["from"])
        m.origin = m["from"]
        m.channel = m.origin
        m.to = m.origin
        m.element = "message"
        m.txt = m["body"]
        m._target = zelf
        if '<delay xmlns="urn:xmpp:delay"' in str(data):
            logging.warn("# Doen/negeer %s %s" % (m.type, m.origin))
            return
        logging.warn("# Doen/%s %s" % (m.type, m.origin))
        zelf._queue.put_nowait(m)

    def handle_presence(zelf, data, *args, **kwargs):
        logging.info("< Doen/lees %s" % data)
        o = Ding(data)
        o["from"] = str(o["from"])
        o.origin = o["from"]
        o.element = "presence"
        if o.type == 'subscribe':
            pres = Ding({'to': o["from"], 'type': 'subscribed'})
            o.doen.send_presence(pres)
            pres = Ding({'to': o["from"], 'type': 'subscribe'})
            o.doen.send_presence(pres)
        elif o.type == "unavailable":
            if o.origin in zelf.channels: zelf.channels.remove(o.origin)
        else:
            if o.origin != zelf.user and o.origin not in zelf.channels: zelf.channels.append(o.origin)
        o.no_dispatch = True
        o._target = zelf
        logging.warn("# Doen/%s %s" % (o.type, o.origin))
        zelf._queue.put_nowait(o)

    def exit(zelf, *args, **kwargs):
        zelf.doen.disconnect()
        Omroeper.exit(zelf)
        zelf._queue.put(None)
        
def init(*args, **kwargs):
    regels = Ding().last("regels", "oor")
    if not regels: regels = regels_doen ; regels.save()
    omroeper = Doen(**regels) 
    vloot.append(omroeper)
    hemel.put(omroeper.loop)   
    hemel.put(omroeper.start)
